@@ -32,9 +32,11 @@
 #include "pilot.h"
 #include "util.h"
 #include "gnome-pilot-pdialog.h"
+#include "gnome-pilot-cdialog.h"
 #include "gnome-pilot-ddialog.h"
 #include "gnome-pilot-capplet.h"
 
+#define SMALL_ICON_SIZE 20
 
 static GtkVBoxClass *parent_class = NULL;
 
@@ -48,6 +50,10 @@ struct _GnomePilotCappletPrivate
 	PilotState *state;
 	PilotState *orig_state;
 
+	GPilotPilot *pilot;
+
+	GHashTable *conduits;
+	
 	CappletWidget *parent;
 
 	GtkWidget *notebook;
@@ -64,6 +70,18 @@ struct _GnomePilotCappletPrivate
 	GtkWidget *devices_delete;
 	GtkWidget *devices_popup;
 
+	GtkWidget *pilots_menu;
+	GtkWidget *pilots_username;
+	GtkWidget *conduit_clist;
+	GtkWidget *conduit_enable;
+	GtkWidget *conduit_disable;
+	GtkWidget *conduit_settings;
+	GtkWidget *conduit_popup;
+	GtkWidget *conduit_popup_enable;
+	GtkWidget *conduit_popup_disable;
+	GtkWidget *conduit_popup_settings;	
+	GtkWidget *conduit_description;
+
 	GtkWidget *sync_pcid;
 };
 
@@ -77,6 +95,7 @@ static void fill_widgets (GnomePilotCapplet *gpcap);
 
 static void check_pilots_buttons (GnomePilotCapplet *gpcap);
 static void check_devices_buttons (GnomePilotCapplet *gpcap);
+static void check_conduits_buttons (GnomePilotCapplet *gpcap);
 
 static void gpcap_pilots_add (GtkWidget *widget, gpointer user_data);
 static void gpcap_pilots_edit (GtkWidget *widget, gpointer user_data);
@@ -90,6 +109,14 @@ static void gpcap_devices_delete (GtkWidget *widget, gpointer user_data);
 static void gpcap_devices_popup (GtkCList *list, GdkEventButton *event, gpointer user_data);
 static void gpcap_devices_selected (GtkCList *clist, gint row, gint col, GdkEventButton *event, gpointer user_data);
 
+static void gpcap_conduits_choose_pilot (GtkWidget *widget, gpointer user_data);
+static void gpcap_conduits_enable (GtkWidget *widget, gpointer user_data);
+static void gpcap_conduits_disable (GtkWidget *widget, gpointer user_data);
+static void gpcap_conduits_settings (GtkWidget *widget, gpointer user_data);
+static void gpcap_conduits_popup (GtkCList *list, GdkEventButton *event, gpointer user_data);
+static void gpcap_conduits_selected (GtkCList *clist, gint row, gint col, GdkEventButton *event, gpointer user_data);
+static void gpcap_conduits_unselected (GtkCList *clist, gint row, gint col, GdkEventButton *event, gpointer user_data);
+
 static void gpcap_cap_try (GtkWidget *widget, gpointer user_data);
 static void gpcap_cap_revert (GtkWidget *widget, gpointer user_data);
 static void gpcap_cap_ok (GtkWidget *widget, gpointer user_data);
@@ -209,6 +236,14 @@ get_widgets (GnomePilotCapplet *gpcap)
 	priv->devices_add = GW ("devices_add_button");
 	priv->devices_edit = GW ("devices_edit_button");
 	priv->devices_delete = GW ("devices_delete_button");
+
+	priv->pilots_menu = GW ("pilots_menu");
+	priv->pilots_username = GW ("username_label");	
+	priv->conduit_clist = GW ("conduit_list");
+	priv->conduit_enable = GW ("conduit_enable_button");
+	priv->conduit_disable = GW ("conduit_disable_button");
+	priv->conduit_settings = GW ("conduit_settings_button");
+	priv->conduit_description = GW ("description_label");
 	
 	priv->sync_pcid = GW ("syncpcid_entry");
 	
@@ -222,6 +257,12 @@ get_widgets (GnomePilotCapplet *gpcap)
 		&& priv->devices_add
 		&& priv->devices_edit
 		&& priv->devices_delete
+		&& priv->pilots_menu
+		&& priv->conduit_clist
+		&& priv->conduit_enable
+		&& priv->conduit_disable
+		&& priv->conduit_settings
+		&& priv->conduit_description		
 		&& priv->sync_pcid);
 }
 
@@ -262,6 +303,14 @@ init_widgets (GnomePilotCapplet *gpcap)
 	gtk_signal_connect (GTK_OBJECT (priv->devices_delete), "clicked",
 			    GTK_SIGNAL_FUNC (gpcap_devices_delete), gpcap);
 
+	gtk_signal_connect (GTK_OBJECT (priv->conduit_enable), "clicked",
+			    GTK_SIGNAL_FUNC (gpcap_conduits_enable), gpcap);
+
+	gtk_signal_connect (GTK_OBJECT (priv->conduit_disable), "clicked",
+			    GTK_SIGNAL_FUNC (gpcap_conduits_disable), gpcap);
+
+	gtk_signal_connect (GTK_OBJECT (priv->conduit_settings), "clicked",
+			    GTK_SIGNAL_FUNC (gpcap_conduits_settings), gpcap);
 	
 	/* Change signals */
 	gtk_signal_connect (GTK_OBJECT (priv->sync_pcid), "changed",
@@ -278,11 +327,18 @@ init_widgets (GnomePilotCapplet *gpcap)
 	gtk_signal_connect (GTK_OBJECT (priv->devices_clist), "unselect-row",
 			    GTK_SIGNAL_FUNC (gpcap_devices_selected), gpcap);
 
+	gtk_signal_connect (GTK_OBJECT (priv->conduit_clist), "select-row",
+			    GTK_SIGNAL_FUNC (gpcap_conduits_selected), gpcap);
+	gtk_signal_connect (GTK_OBJECT (priv->conduit_clist), "unselect-row",
+			    GTK_SIGNAL_FUNC (gpcap_conduits_unselected), gpcap);
+
 	/* Popup menu signals */
 	gtk_signal_connect (GTK_OBJECT (priv->pilots_clist), "button_press_event",
 			    GTK_SIGNAL_FUNC (gpcap_pilots_popup), gpcap);
 	gtk_signal_connect (GTK_OBJECT (priv->devices_clist), "button_press_event",
 			    GTK_SIGNAL_FUNC (gpcap_devices_popup), gpcap);
+	gtk_signal_connect (GTK_OBJECT (priv->conduit_clist), "button_press_event",
+			    GTK_SIGNAL_FUNC (gpcap_conduits_popup), gpcap);
 }
 
 static gint
@@ -365,6 +421,119 @@ fill_devices_clist (GnomePilotCapplet *gpcap)
 	}	
 }
 
+static gint
+append_conduit_clist (GnomePilotCapplet *gpcap, ConduitState *conduit_state)
+{
+	GnomePilotCappletPrivate *priv;
+	GnomePixmap *icon_gpixmap = NULL;
+	GdkPixmap *pmap, *mask;
+	const gchar *row_text[3];
+	gint row = 0;
+	
+	priv = gpcap->priv;
+
+	row_text[0] = conduit_state->name;
+	row_text[1] = display_sync_type_name (conduit_state->enabled,conduit_state->sync_type);
+	row = gtk_clist_append (GTK_CLIST (priv->conduit_clist), (gchar **)row_text);
+
+	if (conduit_state->icon) {
+		icon_gpixmap = (GnomePixmap *)gnome_stock_pixmap_widget_at_size (NULL,
+										 conduit_state->icon,
+										 SMALL_ICON_SIZE,
+										 SMALL_ICON_SIZE);
+	}
+	if (icon_gpixmap) {
+		pmap = icon_gpixmap->pixmap;
+		mask   = icon_gpixmap->mask;
+		gtk_clist_set_pixtext (GTK_CLIST (priv->conduit_clist), row, 0, conduit_state->name, 3, pmap, mask);
+	}
+
+	gtk_clist_set_row_data (GTK_CLIST (priv->conduit_clist), row, conduit_state);
+
+	return row;
+}
+
+static void
+fill_conduit_clist (GnomePilotCapplet *gpcap)
+{
+	GnomePilotCappletPrivate *priv;
+	ConduitState *conduit_state;
+	GList *conduit_states;
+
+	priv = gpcap->priv;
+
+	gtk_clist_clear (GTK_CLIST (priv->conduit_clist));
+
+	if (priv->pilot == NULL)
+		return;
+	
+	if (priv->conduits == NULL )
+		priv->conduits = g_hash_table_new (g_int_hash, g_int_equal);
+	
+	conduit_states = g_hash_table_lookup (priv->conduits, &priv->pilot->pilot_id);
+	if (conduit_states == NULL) {
+		conduit_states = load_conduit_list (priv->pilot);
+		g_hash_table_insert (priv->conduits, &priv->pilot->pilot_id, conduit_states);
+	}
+	
+	while (conduit_states != NULL) {
+		conduit_state = (ConduitState*)conduit_states->data;
+
+		append_conduit_clist (gpcap, conduit_state);
+		conduit_states = conduit_states->next;
+	}
+}
+
+static void
+set_conduit_pilot (GnomePilotCapplet *gpcap, GPilotPilot *pilot)
+{
+	GnomePilotCappletPrivate *priv;
+
+	priv = gpcap->priv;
+
+	priv->pilot = pilot;
+	
+	if (pilot)
+		gtk_label_set_text (GTK_LABEL (priv->pilots_username), pilot->pilot_username);
+	else
+		gtk_label_set_text (GTK_LABEL (priv->pilots_username), "");
+
+	fill_conduit_clist (gpcap);
+	check_conduits_buttons (gpcap);
+}
+
+static void
+fill_pilots_menu (GnomePilotCapplet *gpcap) 
+{
+	GnomePilotCappletPrivate *priv;
+	GtkWidget *menu, *menu_item;
+	GList *tmp;
+	GPilotPilot *pilot = NULL;
+	
+	priv = gpcap->priv;
+	
+	menu = gtk_menu_new ();
+	
+	tmp = priv->state->pilots;
+	while (tmp != NULL) {
+		if (pilot == NULL)
+			pilot = tmp->data;
+		
+		menu_item = gtk_menu_item_new_with_label (((GPilotPilot*)tmp->data)->name);
+		gtk_widget_show (menu_item);
+		gtk_object_set_data (GTK_OBJECT (menu_item), "pilot", tmp->data);
+		gtk_signal_connect (GTK_OBJECT(menu_item),"activate",
+				    GTK_SIGNAL_FUNC (gpcap_conduits_choose_pilot),
+				    gpcap);
+		gtk_menu_append (GTK_MENU (menu), menu_item);
+
+		tmp = tmp->next;
+	}
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (priv->pilots_menu), menu);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (priv->pilots_menu), 0);
+	set_conduit_pilot (gpcap, pilot);
+}
+
 static void
 fill_widgets (GnomePilotCapplet *gpcap)
 {
@@ -381,6 +550,9 @@ fill_widgets (GnomePilotCapplet *gpcap)
 	fill_devices_clist (gpcap);
 	check_devices_buttons (gpcap);
 
+	/* Conduits page */
+	fill_pilots_menu (gpcap);
+
 	/* Advanced page */
 	g_snprintf (buf, sizeof (buf), "%d", priv->state ? priv->state->syncPCid : 0);
 	gtk_entry_set_text (GTK_ENTRY (priv->sync_pcid), buf);
@@ -505,6 +677,8 @@ gpcap_pilots_add (GtkWidget *widget, gpointer user_data)
 		row = append_pilots_clist (gpcap, pilot);
 		gtk_clist_select_row (GTK_CLIST (priv->pilots_clist), row, 0);
 		
+		fill_pilots_menu (gpcap);
+		
 		gpcap_set_changed_state (gpcap);
 	}
 }
@@ -522,7 +696,7 @@ gpcap_pilots_edit (GtkWidget *widget, gpointer user_data)
 	priv = gpcap->priv;
 	
 	pilot = get_current_pilot (gpcap, &row);
-
+	
 	dlg = gnome_pilot_pdialog_new (priv->gpc, priv->state, pilot);
 	res = gnome_pilot_pdialog_run_and_close (GNOME_PILOT_PDIALOG (dlg));
  
@@ -534,6 +708,8 @@ gpcap_pilots_edit (GtkWidget *widget, gpointer user_data)
 		gtk_clist_set_text (GTK_CLIST (priv->pilots_clist), row, 1, buf);
 		gtk_clist_set_text (GTK_CLIST (priv->pilots_clist), row, 2, pilot->pilot_username);
 		gtk_clist_set_text (GTK_CLIST (priv->pilots_clist), row, 3, sync_type_to_str (pilot->sync_options.default_sync_action));
+			
+		fill_pilots_menu (gpcap);
 		
 		gpcap_set_changed_state (gpcap);
 	}
@@ -554,6 +730,9 @@ gpcap_pilots_delete (GtkWidget *widget, gpointer user_data)
 	if (yes_no_dialog (_("Are you sure you want to delete %s pilot?"), pilot->name)) {
 		gtk_clist_remove (GTK_CLIST (priv->pilots_clist), row);
 		priv->state->pilots = g_list_remove (priv->state->pilots, pilot);
+		
+		fill_pilots_menu (gpcap);
+		
 		gpcap_set_changed_state (gpcap);
 	}
 
@@ -729,6 +908,52 @@ gpcap_devices_popup (GtkCList *list, GdkEventButton *event, gpointer user_data)
 	show_popup_menu (list, event, GTK_MENU (priv->pilots_popup));
 }
 
+static ConduitState *
+get_state (GnomePilotCapplet *gpcap, int row)
+{
+	GnomePilotCappletPrivate *priv;
+	
+	priv = gpcap->priv;
+
+	return (ConduitState*)gtk_clist_get_row_data (GTK_CLIST (priv->conduit_clist), row);
+}
+
+
+static ConduitState *
+get_current_state (GnomePilotCapplet *gpcap, int *row)
+{
+	GnomePilotCappletPrivate *priv;
+	gint curr_row;
+	
+	priv = gpcap->priv;
+
+	curr_row = GPOINTER_TO_INT (GTK_CLIST (priv->conduit_clist)->selection->data);
+	if (row)
+		*row = curr_row;
+
+	return get_state (gpcap, curr_row);
+}
+
+static void
+check_conduits_buttons (GnomePilotCapplet *gpcap) 
+{
+	GnomePilotCappletPrivate *priv;
+	ConduitState *state;
+	gboolean test, enabled = FALSE;
+	
+	priv = gpcap->priv;
+
+	test = check_selection (GTK_CLIST (priv->conduit_clist));
+	if (test) {
+		state = get_current_state (gpcap, NULL);
+		enabled = state->enabled;
+	}
+	
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->conduit_enable), test && !enabled);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->conduit_disable), test && enabled);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->conduit_settings), test && enabled);
+}
+
 static void
 gpcap_devices_selected (GtkCList *clist, gint row, gint col, GdkEventButton *event, gpointer user_data)
 {
@@ -737,6 +962,226 @@ gpcap_devices_selected (GtkCList *clist, gint row, gint col, GdkEventButton *eve
 	check_devices_buttons (gpcap);
 }
 
+static void
+gpcap_conduits_choose_pilot (GtkWidget *widget, gpointer user_data)
+{
+	GnomePilotCapplet *gpcap = GNOME_PILOT_CAPPLET (user_data);
+	GnomePilotCappletPrivate *priv;
+	GPilotPilot *pilot;
+	
+	priv = gpcap->priv;
+
+	pilot = gtk_object_get_data (GTK_OBJECT (widget), "pilot");
+
+	set_conduit_pilot (gpcap, pilot);
+}
+
+static void 
+gpcap_conduits_enable (GtkWidget *widget, gpointer user_data)
+{
+	GnomePilotCapplet *gpcap = GNOME_PILOT_CAPPLET (user_data);
+	GnomePilotCappletPrivate *priv;
+	ConduitState *state;
+	gint row;
+	
+	priv = gpcap->priv;
+
+	state = get_current_state (gpcap, &row);
+	
+	if (state->default_sync_type == GnomePilotConduitSyncTypeNotSet) {
+		/* nothing? */
+	} else if (state->default_sync_type == GnomePilotConduitSyncTypeCustom) {
+		state->changed=TRUE;
+		state->enabled=TRUE;
+		state->sync_type = GnomePilotConduitSyncTypeCustom;
+	} else {
+		state->changed = TRUE;
+		state->enabled = TRUE;
+		state->sync_type = state->default_sync_type;
+	}
+
+	gtk_clist_set_text (GTK_CLIST (priv->conduit_clist), row, 1,
+			    display_sync_type_name (state->enabled, state->sync_type));
+
+	check_conduits_buttons (gpcap);
+	gpcap_set_changed_state (gpcap);
+	gpcap_conduits_settings (widget, user_data);
+}
+
+static void 
+gpcap_conduits_disable (GtkWidget *widget, gpointer user_data)
+{
+	GnomePilotCapplet *gpcap = GNOME_PILOT_CAPPLET (user_data);
+	GnomePilotCappletPrivate *priv;
+	ConduitState *state;
+	gint row;
+	
+	priv = gpcap->priv;
+	
+	state = get_current_state (gpcap, &row);
+
+	state->enabled=FALSE;
+	state->changed=TRUE;
+
+	gtk_clist_set_text (GTK_CLIST (priv->conduit_clist), row, 1,
+			    display_sync_type_name (FALSE, GnomePilotConduitSyncTypeNotSet));
+
+	check_conduits_buttons (gpcap);
+	gpcap_set_changed_state (gpcap);
+}
+
+static void 
+gpcap_conduits_settings (GtkWidget *widget, gpointer user_data)
+{
+	GnomePilotCapplet *gpcap = GNOME_PILOT_CAPPLET (user_data);
+	GnomePilotCappletPrivate *priv;
+	ConduitState *state;
+	gint row;
+	
+	priv = gpcap->priv;
+
+	state = get_current_state (gpcap, &row);
+	
+	if (state->conduit == NULL) {
+		/* must allocate conduit */
+		if (gnome_pilot_conduit_management_instantiate_conduit (state->management,
+									state->pilot, 
+									&state->conduit)
+		    != GNOME_PILOT_CONDUIT_MGMT_OK) {
+			
+			gchar *msg = _("Unable to instantiate %s conduit.");
+			error_dialog (msg, state->name);
+			return;
+		}
+
+		state->settings_widget2 = gnome_pilot_cdialog_new (state);
+	}
+	
+	if (state->settings_widget2 != NULL) {
+		if (gnome_pilot_cdialog_run_and_close (GNOME_PILOT_CDIALOG (state->settings_widget2))) {
+			/* pressed ok */
+			state->sync_type = gnome_pilot_cdialog_sync_type (GNOME_PILOT_CDIALOG (state->settings_widget2));
+			state->first_sync_type = gnome_pilot_cdialog_first_sync_type (GNOME_PILOT_CDIALOG (state->settings_widget2));
+			state->enabled = (state->sync_type != GnomePilotConduitSyncTypeNotSet);
+			state->changed=TRUE;
+			
+			gtk_clist_set_text (GTK_CLIST (priv->conduit_clist), row, 1,
+					    display_sync_type_name (state->enabled, state->sync_type));
+			
+			gpcap_set_changed_state (gpcap);
+		} else {
+			/* pressed cancel */
+			gnome_pilot_conduit_display_settings (state->conduit);
+		}
+	}
+	
+	check_conduits_buttons (gpcap);
+}
+
+static void
+gpcap_conduits_popup (GtkCList *list, GdkEventButton *event, gpointer user_data)
+{
+	GnomePilotCapplet *gpcap = GNOME_PILOT_CAPPLET (user_data);
+	GnomePilotCappletPrivate *priv;
+	ConduitState *state;
+	gint row;
+	gboolean test = FALSE, enabled = FALSE;
+	
+	GnomeUIInfo popup [] = {
+		GNOMEUIINFO_ITEM_DATA (N_("_Enable this conduit"),
+				       N_("Disable the currently selected conduit"),
+				       gpcap_conduits_enable, gpcap, NULL),
+		GNOMEUIINFO_ITEM_DATA (N_("_Disable this conduit"),
+				       N_("Enable the currently selected conduit"),
+				       gpcap_conduits_disable, gpcap, NULL),
+
+		GNOMEUIINFO_SEPARATOR,
+
+		GNOMEUIINFO_ITEM_DATA (N_("_Settings..."),
+				       N_("Modify the currently selected conduit's settings"),
+				       gpcap_conduits_settings, gpcap, NULL),
+		
+		GNOMEUIINFO_END
+	};
+ 
+	priv = gpcap->priv;
+
+	if (priv->conduit_popup == NULL) {
+		priv->conduit_popup = gtk_menu_new ();
+		gnome_app_fill_menu (GTK_MENU_SHELL (priv->conduit_popup), popup, NULL, TRUE, 0);
+
+		priv->conduit_popup_enable = popup[0].widget;
+		priv->conduit_popup_disable = popup[1].widget;
+		priv->conduit_popup_settings = popup[3].widget;
+		
+	}
+	
+	if (gtk_clist_get_selection_info (list, event->x, event->y, &row, NULL)) {
+		state = get_state (gpcap, row);
+		test = TRUE;
+		enabled = state->enabled;
+	}
+
+	gtk_widget_set_sensitive (priv->conduit_popup_enable, test && !enabled);
+	gtk_widget_set_sensitive (priv->conduit_popup_disable, test && enabled);
+	gtk_widget_set_sensitive (priv->conduit_popup_settings, test && enabled);
+
+	show_popup_menu (list, event, GTK_MENU (priv->conduit_popup));
+}
+
+static void
+gpcap_conduits_selected (GtkCList *clist, gint row, gint col, GdkEventButton *event, gpointer user_data)
+{
+	GnomePilotCapplet *gpcap = GNOME_PILOT_CAPPLET (user_data);
+	GnomePilotCappletPrivate *priv;
+	ConduitState *state;
+	
+	priv = gpcap->priv;
+	
+	state = get_current_state (gpcap, NULL);
+	
+	gtk_label_set_text (GTK_LABEL (priv->conduit_description), state->description);
+	check_conduits_buttons (gpcap);
+}
+
+static void
+gpcap_conduits_unselected (GtkCList *clist, gint row, gint col, GdkEventButton *event, gpointer user_data)
+{
+	GnomePilotCapplet *gpcap = GNOME_PILOT_CAPPLET (user_data);
+	
+	check_conduits_buttons (gpcap);
+}
+
+static void 
+conduit_try_foreach (gpointer key, gpointer value, gpointer func)
+{
+	GList *states = (GList*)value;
+	
+	while (states != NULL) {
+		ConduitState *state = (ConduitState*)states->data;
+		
+		if (state->changed && state->enabled) {
+			if (state->sync_type == GnomePilotConduitSyncTypeCustom)
+				gnome_pilot_conduit_config_enable (state->config, GnomePilotConduitSyncTypeCustom);
+			else
+				gnome_pilot_conduit_config_enable_with_first_sync (state->config,
+										   state->sync_type,
+										   state->first_sync_type,
+										   TRUE);
+			if (state->first_sync_type == GnomePilotConduitSyncTypeNotSet)
+				gnome_pilot_conduit_config_remove_first_sync (state->config);
+		}
+		
+		if (state->changed && !state->enabled)
+			gnome_pilot_conduit_config_disable (state->config);
+		
+		if (state->conduit && state->changed)
+			gnome_pilot_conduit_save_settings (state->conduit);
+		
+		states = states->next;
+	}
+}
+
 static void 
 gpcap_cap_try (GtkWidget *widget, gpointer user_data)
 {
@@ -755,9 +1200,45 @@ gpcap_cap_try (GtkWidget *widget, gpointer user_data)
 		tmp = tmp->next;
 	}
 
+	g_hash_table_foreach (priv->conduits, conduit_try_foreach, NULL);
+	fill_conduit_clist (gpcap);
+	fill_pilots_menu (gpcap);
+	
 	save_config_and_restart (priv->gpc, priv->state);
 }
 
+static void
+conduit_revert_foreach (gpointer key, gpointer value, gpointer func)
+{
+	GList *states = (GList*)value;
+	
+	while (states != NULL) {
+		ConduitState *state = (ConduitState*)states->data;
+		
+		if (state->changed && state->orig_enabled) {
+			if (state->orig_sync_type == GnomePilotConduitSyncTypeCustom)
+				gnome_pilot_conduit_config_enable (state->config, GnomePilotConduitSyncTypeCustom);
+			else
+				 gnome_pilot_conduit_config_enable (state->config, state->orig_sync_type);
+			
+			state->enabled = state->orig_enabled;
+			state->sync_type = state->orig_sync_type;
+			state->first_sync_type = state->orig_first_sync_type;
+		}
+		
+		if (state->changed && !state->orig_enabled) {
+			gnome_pilot_conduit_config_disable (state->config);
+			
+			state->enabled = state->orig_enabled;
+		}
+		
+		if (state->conduit && state->changed)
+			gnome_pilot_conduit_revert_settings (state->conduit);
+		
+		states = states->next;
+	}
+}
+
 static void
 gpcap_cap_revert (GtkWidget *widget, gpointer user_data)
 {
@@ -765,11 +1246,14 @@ gpcap_cap_revert (GtkWidget *widget, gpointer user_data)
 	GnomePilotCappletPrivate *priv;
 	
 	priv = gpcap->priv;
-	
+
+	g_hash_table_foreach (priv->conduits, conduit_revert_foreach, gpcap);
+	fill_conduit_clist (gpcap);
+
 	save_config_and_restart (priv->gpc, priv->orig_state);
 	freePilotState (priv->state);
 	priv->state = dupPilotState (priv->orig_state);
-
+	
 	fill_widgets (gpcap);
 }
 
@@ -804,6 +1288,8 @@ gpcap_destroy (GtkObject *object)
 	freePilotState (priv->orig_state);
 	freePilotState (priv->state);
 
+	g_hash_table_destroy (priv->conduits);
+
 	gtk_object_unref (GTK_OBJECT (priv->xml));
 	
 	if (priv->pilots_popup)
@@ -811,7 +1297,10 @@ gpcap_destroy (GtkObject *object)
 
 	if (priv->devices_popup)
 		gtk_widget_destroy (priv->devices_popup);
-	
+
+	if (priv->conduit_popup)
+		gtk_widget_destroy (priv->conduit_popup);
+
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
