@@ -119,14 +119,32 @@ static int
 pilot_connect (GPilotDevice *device,int *error) 
 {
 	struct pi_sockaddr addr;
-	int sd, listen_sd;
+	int sd, listen_sd, pf;
 	int ret;
     
 	if (device->type != PILOT_DEVICE_NETWORK) {
 		pilot_set_baud_rate (device);
 	}
 
-	if (!(listen_sd = pi_socket (PI_AF_PILOT, PI_SOCK_STREAM, PI_PF_DLP))) {
+	switch (device->type) {
+	case PILOT_DEVICE_SERIAL:
+		pf = PI_PF_PADP;
+		break;
+	case PILOT_DEVICE_USB_VISOR:
+		pf = PI_PF_NET;
+		break;
+	case PILOT_DEVICE_IRDA:
+		pf = PI_PF_PADP;
+		break;
+	case PILOT_DEVICE_NETWORK:
+		pf = PI_PF_NET;
+		break;
+	default:
+		pf = PI_PF_DLP;
+		break;
+	}
+	
+	if (!(listen_sd = pi_socket (PI_AF_PILOT, PI_SOCK_STREAM, pf))) {
 		g_warning ("pi_socket: %s",strerror (errno));
 		if (error) *error = 1;
 		return -1;
